
/*    ///RAVE UNIT_///
// |**********************************************************************;
* Project           : Replacement solid RPM Rave 7800 or 8400 unit, for use with the stock Solenoid
*                   
* 
* Program name      : RAVE_Unit_Micros.ino
*
* Author            : InterlinkKnight
*
* Date created      : 05/23/2019
*
* Modified by       : el bodo es loco
*
* Revision History  :
*
* Date modifiied    Vers.   Revision Date  
* 11/06/2023         2.0     11/06/2023 (mm/dd/jjjj)
*
|**********************************************************************;
*/
#include <Wire.h>

#define CONTROL_PIN 9  // pwm to the soleniod
#define LED_PIN 13 // rave light 






//##### Kalibrierung / Calibration #####// 
//########################//  Wichtige Parameter sind mit "####" gekennzeichnet // important values are signed with "####" 

int set = 8200 ;  //#### einstellung der öffnungsdrehzahl //set up the rotations per minute for opening the power valve
bool valve_cleaning = true; // false; // deavtiviert die reinigungsfunktion,  true; // aktiviert die reinigunsfunktion // powervalve clenaning function while idle:, true = activated, false is deactivated
const long interval2 = 2000; //#### einstellung der Wartezeit in Millisekunden bis die nervige reinigungsfunktion aktiviert wird //set up a delay in milliseconds for valve cleaning function  
// to prevent a power loss by valve cleaning while drive off 
int setsmooth = 100; // a value of 100 means: opening valve at 8200 rpm and close the valve at 8100 
const byte PulsesPerRevolution = 6;  // #### 1 für Drehzahlmessung am Pickup oder Zündspule; 6 für Drehzahlmessung an der Lima (ROTAX)
//  1 is for rpm reading on Pickup or Ignition coil (ROTAX 122), 6 is for reading on alternator (ROTAX 122 and ROTAX 123)  





//Timeout und Genauigkeit// 
const unsigned long ZeroTimeout = 100000;  
const byte numReadings = 30;  //Anzahl der Messwertblöcke zur Mittelwert bildung Je höher des to genauer aber auch trägerzwischen 3-10 ist in Ordnung
//muss nicht verändert werden 

/////////////
// Variablen:
/////////////

volatile unsigned long LastTimeWeMeasured;  
volatile unsigned long PeriodBetweenPulses = ZeroTimeout+1000;  
volatile unsigned long PeriodAverage = ZeroTimeout+1000;  
unsigned long FrequencyRaw;
unsigned long previousMillis = 0; 
unsigned long FrequencyReal; 
unsigned long RPM; 
unsigned int PulseCounter = 1; 
const int PAUSE = 3000;

unsigned long CurrentMillis = 0;
unsigned long cut_off_timer = 0;
const int32_t cut_off = 20 ; 
unsigned long rave_timer = 0;
unsigned long idle_timer = 0;
const int32_t rave = 1000 ;
unsigned long timer1 = 0;


unsigned long PeriodSum; 
unsigned long LastTimeCycleMeasure = LastTimeWeMeasured;  
unsigned long CurrentMicros = micros(); 
unsigned int AmountOfReadings = 1;
unsigned int ZeroDebouncingExtra;  
unsigned long readings[numReadings];  
unsigned long readIndex;  
unsigned long total;  
unsigned long average;  

void setup() 
{

  
  
  Wire.begin(); 

  attachInterrupt(digitalPinToInterrupt(3), Pulse_Event, FALLING);  
  pinMode (3, INPUT_PULLUP);

  pinMode (LED_PIN, OUTPUT);
  pinMode (CONTROL_PIN, OUTPUT);
  analogWrite (CONTROL_PIN, 0);
      
}  

void loop()
{
LastTimeCycleMeasure = LastTimeWeMeasured;
CurrentMicros = micros(); 


if(CurrentMicros < LastTimeCycleMeasure)
  {
    LastTimeCycleMeasure = CurrentMicros;
  }

  FrequencyRaw = 10000000000 / PeriodAverage;
  if(PeriodBetweenPulses > ZeroTimeout - ZeroDebouncingExtra || CurrentMicros - LastTimeCycleMeasure > ZeroTimeout - ZeroDebouncingExtra)
  {  
    FrequencyRaw = 0;   
    ZeroDebouncingExtra = 2000;
  }
  else
  {
    ZeroDebouncingExtra = 0;  
  }
 FrequencyReal = FrequencyRaw / 10000;  
  RPM = FrequencyRaw / PulsesPerRevolution * 60;  
  RPM = RPM / 10000; 
  total = total - readings[readIndex];  
  readings[readIndex] = RPM;  
  total = total + readings[readIndex];  
  readIndex = readIndex + 1;  

  if (readIndex >= numReadings) 
  {
    readIndex = 0;  
  }
  average = total / numReadings; 
int r = RPM ;

if (RPM <= 100){ // unter diesem Wert bleibt der Auslassschieber geschlossen
        digitalWrite(LED_PIN,LOW);
     analogWrite (CONTROL_PIN, 0);
}

  if (RPM <= 2300 && RPM >= 100 && valve_cleaning ){ // unter diesem wert ist die Reinugungsfunktion aktiv
        timer1 = millis ();
        Idle_Event();
  }

    if (RPM <= set-setsmooth && RPM >= 2300){ // unter diesem Wert bleibt der Auslassschieber geschlossen
        digitalWrite(LED_PIN,LOW);
       CurrentMillis = millis ();
           analogWrite (CONTROL_PIN, 0);
        
   }

        if (RPM >= set){ // über diesem Wert öffnet der Auslassschieber
           
         Rave_Event();
         
       }

  
}  
void Pulse_Event()  
{

  PeriodBetweenPulses = micros() - LastTimeWeMeasured;  

  LastTimeWeMeasured = micros();  

 if(PulseCounter >= AmountOfReadings)  
  {
    PeriodAverage = PeriodSum / AmountOfReadings; 
    PulseCounter = 1;  
    PeriodSum = PeriodBetweenPulses; 
    int RemapedAmountOfReadings = map(PeriodBetweenPulses, 40000, 5000, 1, 10);  
    
    RemapedAmountOfReadings = constrain(RemapedAmountOfReadings, 1, 10);
    AmountOfReadings = RemapedAmountOfReadings;  
  }
  else
  {
    PulseCounter++; 
    PeriodSum = PeriodSum + PeriodBetweenPulses; 
  }

}  



void Rave_Event(){
rave_timer = millis ();
digitalWrite (LED_PIN, HIGH);
  
  if (rave_timer - CurrentMillis  <= rave){
    analogWrite (CONTROL_PIN, 255);
  }

  if (rave_timer - CurrentMillis  >= rave){
    analogWrite (CONTROL_PIN, 102);
  } 
 }


void Idle_Event(){
idle_timer = millis ();
digitalWrite (LED_PIN, HIGH);
if (idle_timer - CurrentMillis  <= interval2){
    analogWrite (CONTROL_PIN, 0);
  }

  
  if (idle_timer - CurrentMillis  >= interval2 && idle_timer - CurrentMillis  <= rave+ interval2){
    analogWrite (CONTROL_PIN, 255);
  }

  if (idle_timer - CurrentMillis  >= interval2 && idle_timer - CurrentMillis  >= rave+ interval2){
    analogWrite (CONTROL_PIN, 102);
  } 
 }


  
